#####################################################################################################################
# Copyright IBM 2008 Stuart Poulin
#
#
#!ratlperl -l -w
my $Version = "7.0.1.0.01";
use strict; 
use vars qw($Bin);
use FindBin qw($Bin);
use lib "$Bin";
use Getopt::Long;
use File::Basename;
use Carp;
use CQPerlExt;
use Env;
my $progname = basename $0;

# Unbuffer STDERR and STDOUT
select STDERR;  
$| = 1;
select STDOUT;
$| = 1;

my $progname = basename $0;


#####
# Subroutine to display the usage
sub usage {
    print STDERR <<"!E!O!F!";
    Version=$Version

    Usage: $progname
    -user <user>  The login id of the ClearQuest user.
    -pw    The password of the ClearQuest user.
    -dbname  The schema repository connection to the database.
    -dbset    The user database connection.
    -query <querydef>  The query to call.
    -show              This will display all the parameters in the query.
                       Use this to set the -param,-cmp,-value as follows.
    -out <output_file> Save output of query to file.


    Note the following must be supplied at triplets.  
    If you supply -param, you must supply a -cmp and -value.
    -param <number|name>    The number of a parameter, or the exact name
                            to filter
    -cmp <OP>          The "Compare" operator,  i.e "LIKE", "GT", "GTE"
    -value <value>     The value to compare against.

    Addtional options
    -help  This is a flag for the help.

    Examples: 
    ratlperl $progname ^
	-query "Personal Queries/non-UCM Delivered Activities" ^
	-param 2 -cmp LIKE -value "Dev" ^
	-param 1 -cmp GT -value "Feb 1, 2008"

    ratlperl $progname ^
	-query "Personal Queries/non-UCM Delivered Activities" ^
	-param "type" -cmp LIKE -value "Dev" ^
	-param action_timestamp -cmp GT -value "Feb 1, 2008"

    ratlperl $progname ^
	-query "Personal Queries/non-UCM Delivered Activities" -show 
!E!O!F!
exit 1;
}

####
# Define the input parameters
my @opts = qw(
    user=s		
    pw=s		
    dbname=s		
    dbset=s		
    query=s	
    param=s@
    cmp=s@
    value=s@
    output=s
    show
    help
);

my %opts = (
    user => $ENV{ALMUSER},
    pw => $ENV{ALMPW},
    dbname => $ENV{ALMDBNAME},
    dbset => $ENV{ALMDBSET},
    query => undef,
    show=>0,
    param=>[],
    cmp=>[],
    value=>[],
    output=>"",
);

# Windows workaround to allow empty passwords for query/read-only users
$opts{pw} = "" if ($opts{pw} eq q{""});


my @required_opts = qw ( user pw dbname dbset query);

####
# Get the input parameters
&usage unless (GetOptions(\%opts, @opts)) ;

{my  $message = "";
	for my $req (@required_opts )  {
		$message .= "Missing required option -$req \n" if ! defined $opts{$req};
	}
	if ($message ) {
		print STDERR $message,"\n";
		&usage();
	}
}
;

#params=>[{PARAM=>2,CMP=>$CQPerlExt::CQ_COMP_OP_LIKE,VALUE=>"J"},],
my @params;
for (my $i = 0 ; $i < scalar @{$opts{param}}  ; $i++ ){
	my $param = $opts{param}[$i];
	my $cmp = $opts{cmp}[$i]; uc $cmp;
	$cmp = eval "\$CQPerlExt::CQ_COMP_OP_${cmp}";
	my $value = $opts{value}[$i];
	push @params,{PARAM=>$param,CMP=>$cmp,VALUE=>$value};
}


&usage if ($opts{help});

if ($opts{output}) {
	open(OUT,">$opts{output}") or die "$progname: >$opts{output} J܂";
	open(STDOUT, ">&OUT") or die "$progname: STDOUT _CNgł܂";
}


####
#### Start a CQ session
my $session = CQPerlExt::CQSession_Build();
#### Logon to the specified database

eval {$session->UserLogon("$opts{user}", "$opts{pw}", "$opts{dbname}", "$opts{dbset}")};

if ($@ or ! defined $session ) {
	print STDERR $@;
	exit 1 ;
}


my $workspace = $session->GetWorkSpace();
my $querydef = $workspace->GetQueryDef($opts{query});

if ($opts{show}){
	ShowParams(session=>$session,querydef=>$querydef);
}
else {
	my ($params,$syms) = FillParams(session=>$session,querydef=>$querydef);
	# Look up symbolic names if given.
	for my $p (@params) {
		if($p->{PARAM} !~ /^\d+$/ ) { # An integer
		    my $nu = $syms->{$p->{PARAM}};
		    $p = {PARAM=>$nu,CMP=>$p->{CMP},VALUE=>$p->{VALUE}};
		}
	}
	RunBasicQuery(session=>$session,querydef=>$querydef,
		params=>\@params, start_column=>1
	);
}



sub RunBasicQuery {
	my %lopts = @_;
	my $session=$lopts{session};
	my $querydef=$lopts{querydef};
	#  The parameters to this subroutine are a Session object 
	#  and a QueryDef object. It is assumed that the QueryDef 
	#  is valid (for example, BuildField has been used to select 
	#  one or more fields to retrieve). 

	my ($rsltset);      #  This is a ResultSet object 
	my ($status);
	my ($column);
	my ($num_columns);
	my ($num_records);

	$rsltset = $lopts{session}->BuildResultSet($lopts{querydef});

	for my $p (@{$lopts{params}}) {
			my $param = $p->{PARAM};
			my $value = $p->{VALUE};
			my $cmp = $p->{CMP};
			$rsltset->AddParamValue($param, $value);
			$rsltset->SetParamComparisonOperator($param,$cmp);
	}

	$rsltset->Execute;

	$rsltset->LookupPrimaryEntityDefName;

	   $num_columns = $rsltset->GetNumberOfColumns;
	   $num_records = 0;
	   $status = $rsltset->MoveNext;
	   my $first = 0;
	   while ($status == $CQPerlExt::CQ_SUCCESS) {
	       if ( ! $first ) {
		       $first++;
		      $column = 1 + $lopts{start_column};
		      print "#";
		      while ($column <= $num_columns) {
			 print " ", $rsltset->GetColumnLabel($column), "," ;
			 $column = $column + 1;
		      }
		      print "\n";

	       }

	      #  Note: result set indices are based 1..N, not the usual
	      # 0..N-1 
	      $column = 1 + $lopts{start_column};
	      while ($column <= $num_columns) {
		 print $rsltset->GetColumnValue($column),",";
		 $column = $column + 1;
	      }
	      print "\n";
	      $status = $rsltset->MoveNext;
	   }
} 

sub ShowParams {
	my %lopts = @_;
	my $session=$lopts{session};
	my $querydef=$lopts{querydef};
	#  The parameters to this subroutine are a Session object 
	#  and a QueryDef object. It is assumed that the QueryDef 
	#  is valid (for example, BuildField has been used to select 
	#  one or more fields to retrieve). 

	my ($rsltset);      #  This is a ResultSet object 

	$rsltset = $session->BuildResultSet($querydef);
	my $methods = [
		{f=>"GetParamLabel",r=>"value"},
		{f=>"GetParamComparisonOperator", r=>"value"},
		{f=>"GetParamFieldType",r=>"value" },
		{f=>"GetParamPrompt",r=>"value" },
		{f=>"GetParamChoiceList",r=>"array" },
	];

	my $c = $rsltset->GetNumberOfParams();
	print "GetNumberOfParams:$c\n";
	for (my $i = 1 ; $i <= $c; $i++) {
		print "\tParam:$i\n";
		for my $m (@$methods) {
			my $method = $m->{f};
			my $return = $m->{r};
			if ( $return =~ /array/) {
				print "\t\t$method:\n";
				my $choices = $rsltset->$method($i);
				for my $v (@$choices) {
					print "\t\t\t$v\n";
				}
			}
			else {
				my $result = $rsltset->$method($i);
				print "\t\t$method:$result\n";
			}
			print "\n";
		}
	}

}

sub FillParams {
	my %lopts = @_;
	my $session=$lopts{session};
	my $querydef=$lopts{querydef};
	my $params =[];
	my $symbolic={};
	#  The parameters to this subroutine are a Session object 
	#  and a QueryDef object. It is assumed that the QueryDef 
	#  is valid (for example, BuildField has been used to select 
	#  one or more fields to retrieve). 

	my ($rsltset);      #  This is a ResultSet object 

	$rsltset = $session->BuildResultSet($querydef);
	my $methods = [
		{f=>"GetParamLabel",r=>"value"},
		{f=>"GetParamComparisonOperator", r=>"value"},
		{f=>"GetParamFieldType",r=>"value" },
		{f=>"GetParamPrompt",r=>"value" },
		{f=>"GetParamChoiceList",r=>"array" },
	];

	my $c = $rsltset->GetNumberOfParams();
	
	for (my $i = 1 ; $i <= $c; $i++) {
		$params->[$i]->{value} = $i;
		for my $m (@$methods) {
			my $method = $m->{f};
			my $return = $m->{r};
			if ( $return =~ /array/) {
				my $choices = $rsltset->$method($i);
				for my $v (@$choices) {
					push @{$params->[$i]->{choices}},$v;
				}
			}
			else {
				my $result = $rsltset->$method($i);
				$params->[$i]->{$method} = $result;
				if ($method eq "GetParamLabel" ) {
					$symbolic->{$result} = $i;
				}
			}
		}
	}
	return ($params,$symbolic);

}
